@@ -1,19 +1,24 @@
-/***************************************************************************
-                          dvbpanel.cpp  -  description
-                             -------------------
-    begin                : Mon Jan 19 2004
-    copyright            : (C) 2004-2006 by Christophe Thommeret
-    email                : hftom@free.fr
- ***************************************************************************/
+/*
+ * dvbpanel.cpp
+ *
+ * Copyright (C) 2004-2006 Christophe Thommeret <hftom@free.fr>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+ */
 
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+#include <sys/statvfs.h>
 
 #include <qlayout.h>
 #include <qfile.h>
@@ -35,8 +40,13 @@
 #include <kicondialog.h>
 #include <kaction.h>
 #include <kprocess.h>
+#include <kfiledialog.h>
+#include <ktrader.h>
+#include <kxmlguifactory.h>
+#include <kparts/componentfactory.h>
 
 #include "kaffeineinput.h"
+#include "kaffeinedvbplugin.h"
 #include "dvbpanel.h"
 #include "channeldesc.h"
 #include "dvbstream.h"
@@ -44,6 +54,8 @@
 #include "kevents.h"
 #include "broadcasteditor.h"
 
+#define CHANICONSIZE 28
+
 
 
 DIconViewItem::DIconViewItem( DvbPanel *pan, QIconView *parent, const QString &text, const QPixmap &icon )
@@ -85,7 +97,9 @@ QDragObject* DListView::dragObject()
 DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : KaffeineInput(objParent,name)
 {
 	browseDvbStream = -1;
+	plug = NULL;
 
+	isTuning = false;
 	timeShiftFileName = "";
 	timersDialog = 0;
 	currentCategory = "All";
@@ -116,7 +130,6 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 	panel->setLineWidth(1);
 	panel->setFrameStyle(QFrame::Panel|QFrame::Sunken);
 	split->moveToFirst( panel );
-	split->moveToLast( pbox );
 	split->setResizeMode( panel, QSplitter::KeepSize );
 
 	QVBoxLayout *vb = new QVBoxLayout( panel, 3, 3 );
@@ -150,6 +163,7 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 	//channelsCb->setPaletteBackgroundColor( QColor(255,255,200) );
 	connect( channelsCb, SIGNAL(itemRenamed(QListViewItem*)), this, SLOT(channelNumberChanged(QListViewItem*)) );
 	connect( channelsCb, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(channelSelected(QListViewItem*)) );
+	connect( channelsCb, SIGNAL(contextMenuRequested(QListViewItem*,const QPoint&,int)), this, SLOT(channelPopup(QListViewItem*,const QPoint&,int)) );
 	channelsCb->setItemsRenameable( true );
 	channelsCb->addColumn( i18n("Number") );
 	channelsCb->addColumn( i18n("Name") );
@@ -233,6 +247,9 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 	connect( &timersTimer, SIGNAL(timeout()), this, SLOT(checkTimers()) );
 	connect( &osdTimer, SIGNAL(timeout()), this, SLOT(resetOSD()) );
 	connect( &showOsdTimer, SIGNAL(timeout()), this, SLOT(dvbOSD()) );
+	connect( &tuningTimer, SIGNAL(timeout()), this, SLOT(setTuning()) );
+	connect( &stopTuningTimer, SIGNAL(timeout()), this, SLOT(setTuning()) );
+	connect( &diskTimer, SIGNAL(timeout()), this, SLOT(diskStatus()) );
 
 	setConfig();
 
@@ -243,6 +260,70 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 
 
 
+void DvbPanel::diskStatus()
+{
+	double freemb;
+	struct statvfs buf;
+
+	if ( statvfs( dvbConfig->shiftDir.local8Bit(), &buf ) ) {
+		fprintf(stderr,"Couldn't get file system statistics\n");
+		return;
+	}
+	
+	freemb = (double)(((double)(buf.f_bavail)*(double)(buf.f_bsize))/(1024.0*1024.0));
+	if ( freemb<300 )
+		emit showOSD( i18n("Warning: low disc space")+QString(" (%1").arg((int)freemb)+i18n("MB")+")", 5000, 3 );
+}
+
+
+
+void DvbPanel::channelPopup( QListViewItem *it, const QPoint &pos, int col )
+{
+	QPoint p=pos;
+
+	if ( !it )
+		return;
+
+	int i=col, j;
+	QImage img;
+	QPixmap pix;
+	QString name = it->text(1);
+	QString s;
+
+	QPopupMenu *pop = new QPopupMenu();
+	pop->insertItem( i18n("Select icon..."), 1 );
+	i = 0;
+	i = pop->exec( QCursor::pos() );
+	switch ( i ) {
+		case 0 :
+			break;
+		case 1 :
+			s = KFileDialog::getOpenFileName( QString::null, "image/png image/jpeg image/gif image/x-bmp image/x-xpm", 0, i18n("Choose channel icon"));
+			if ( s.isEmpty() )
+				break;
+			img = QImage( s );
+			if ( img.isNull() )
+				break;
+			if ( img.width()>img.height() )
+				img = img.smoothScale( CHANICONSIZE, img.height()*CHANICONSIZE/img.width() );
+			else
+				img = img.smoothScale( img.width()*CHANICONSIZE/img.height(), CHANICONSIZE );
+			pix.convertFromImage( img );
+			for ( j=0; j<(int)channels.count(); j++ ) {
+				if ( channels.at(j)->name==name ) {
+					channels.at(j)->pix = pix;
+					break;
+				}
+			}
+			pix.save( dvbConfig->dvbConfigIconsDir+name, "PNG" );
+			fillChannelList();
+			break;
+	}
+	delete pop;
+}
+
+
+
 void DvbPanel::getTargets( QStringList &uiNames, QStringList &iconNames, QStringList &targetNames )
 {
 	uiNames.append( i18n("Digital TV") );
@@ -268,28 +349,6 @@ bool DvbPanel::execTarget( const QString &target )
 
 
 
-void DvbPanel::toggleLayout( bool b )
-{
-	QValueList<int> sizes = split->sizes();
-	int i = sizes[0];
-	sizes[0] = sizes[1];
-	sizes[1] = i;
-
-	if ( b ) {
-		split->moveToLast( panel );
-		split->moveToFirst( pbox );
-		split->setSizes( sizes );
-	}
-	else {
-		split->moveToLast( pbox );
-		split->moveToFirst( panel );
-		split->setSizes( sizes );
-	}
-	dvbConfig->alternateLayout = b;
-}
-
-
-
 void DvbPanel::setupActions()
 {
 	new KAction(i18n("OSD Next Channel"), "next", CTRL + SHIFT + Key_W, this, SLOT(dvbOSDNext()), actionCollection(), "dvb_browse_next");
@@ -674,6 +733,7 @@ void DvbPanel::dvbOSD(ChannelDesc liveChannel, DvbStream *d, int timeShift /* =
 	QPtrList<EventDesc> *currentEvents=0;
 	QString s;
 	int i=0, j, k;
+	int first=0;
 
 	if ( !d )
 		return;
@@ -757,6 +817,17 @@ void DvbPanel::dvbOSD(ChannelDesc liveChannel, DvbStream *d, int timeShift /* =
 			s = desc->startDateTime.toString( "hh:mm" );
 			s = s+" - ";
 			s = s+desc->title;
+			if ( !osdMode && !first && !myshift ) {
+				QDateTime dt = QDateTime::currentDateTime();
+				int secs = desc->startDateTime.secsTo( dt );
+				i = QTime().secsTo( desc->duration );
+				if ( i!=0 )
+					i = secs*100/i;
+				if ( i>100 )
+					i = 100;
+				list.append( QString("BAR")+QString().setNum( i ) );
+				++first;
+			}
 			list.append( s );
 			k++;
 		}
@@ -773,13 +844,13 @@ void DvbPanel::dvbOSD(ChannelDesc liveChannel, DvbStream *d, int timeShift /* =
 		for ( i=0; i<(int)desc->shortEvents.count(); i++ ) {
 			s = s + desc->shortEvents.at(i)->name;
 			if (!desc->shortEvents.at(i)->name.isEmpty() && !desc->shortEvents.at(i)->text.isEmpty()) {
-				s = s + " : ";
-				s = s + desc->shortEvents.at(i)->text;
-				s = s+". ";
+				s+= " : ";
+				s+= desc->shortEvents.at(i)->text;
+				s+= ". ";
 			}
 		}
 		for ( i=0; i<(int)desc->extEvents.count(); i++ )
-			s = s+ *desc->extEvents.at(i);
+			s+= *desc->extEvents.at(i);
 		list.append( s );
 		break;
 	}
@@ -811,6 +882,7 @@ void DvbPanel::checkFirstRun()
 void DvbPanel::setConfig()
 {
 	int i;
+	int error = 0;
 	KIconViewItem* item = NULL;
 	DvbStream *d;
 
@@ -823,23 +895,40 @@ void DvbPanel::setConfig()
 	item->setDropEnabled( false );
 	item = new DIconViewItem(this, iconView, i18n("TV"), KGlobal::iconLoader()->loadIcon(dvbConfig->tvIcon, KIcon::NoGroup, KIcon::SizeSmallMedium));
 	item->setDropEnabled( false );
+
+	KTrader::OfferList offers = KTrader::self()->query("KaffeineDvbPlugin");
+	KTrader::OfferList::Iterator it = offers.begin();
+	if ( it!=offers.end() ) {
+		KService::Ptr ptr = (*it);
+		plugName = ptr->desktopEntryName();
+		plug = KParts::ComponentFactory::createPartInstanceFromService<KaffeineDvbPlugin>(ptr, 0, ptr->name().ascii(), 0, 0, 0, &error);
+		if (error > 0) {
+			fprintf( stderr, "Loading of DVB plugin failed: %s\n", KLibLoader::self()->lastErrorMessage().ascii() );
+			plug = NULL;
+		}
+		else {
+			fprintf( stderr, "DVB plugin loaded\n.");
+		}
+	}
+
 	for ( i=0; i<(int)dvbConfig->categories.count(); i++ ) {
 		item = new DIconViewItem(this, iconView, dvbConfig->categories.at(i)->name, KGlobal::iconLoader()->loadIcon(dvbConfig->categories.at(i)->icon, KIcon::NoGroup, KIcon::SizeSmallMedium));
 		item->setDropEnabled( true );
 	}
 
 	for ( i=0; i<(int)dvbConfig->devList.count(); i++ ) {
-		d = new DvbStream( dvbConfig->devList.at(i)->adapter, dvbConfig->devList.at(i)->tuner, dvbConfig->devList.at(i)->type, dvbConfig->devList.at(i)->source, dvbConfig->defaultCharset );
+		d = new DvbStream( dvbConfig->devList.at(i)->adapter, dvbConfig->devList.at(i)->tuner, dvbConfig->devList.at(i)->type, dvbConfig->devList.at(i)->source, dvbConfig->devList.at(i)->canAutoscan, dvbConfig->defaultCharset );
 		d->setLnb( 0, dvbConfig->devList.at(i)->lnb0 );
 		d->setLnb( 1, dvbConfig->devList.at(i)->lnb1 );
 		d->setLnb( 2, dvbConfig->devList.at(i)->lnb2 );
 		d->setLnb( 3, dvbConfig->devList.at(i)->lnb3 );
+		d->setPlug( plug );
 		dvb.append( d );
 		connect( d, SIGNAL(shifting(bool)), this, SLOT(setShiftLed(bool)) );
 		connect( d, SIGNAL(isBroadcasting(bool)), this, SLOT(setBroadcastLed(bool)) );
 		connect( d, SIGNAL(timerEnded(RecTimer*)), this, SLOT(killTimer(RecTimer*)) );
 		connect( d, SIGNAL(isRecording(bool)), this, SLOT(setRecordLed(bool)) );
-		connect( d, SIGNAL(playDvb()), this, SIGNAL(playDvb()) );
+		connect( d, SIGNAL(playDvb()), this, SLOT(pipeOpened()) );
 	}
 	fifoName = QDir::homeDirPath()+"/.kaxtv.ts";
 	QFile f( fifoName );
@@ -859,15 +948,11 @@ void DvbPanel::setConfig()
 	}
 	currentFifo = fifoName;
 	getTimerList();
-	timersTimer.start( 1000 );
+	timersTimer.start( 5000 );
 	getChannelList();
 	rtp = new Ts2Rtp();
 	rtp->setSocket( dvbConfig->broadcastAddress, dvbConfig->broadcastPort, dvbConfig->senderPort );
 	cleaner = new Cleaner( dvbConfig->shiftDir );
-	if ( dvbConfig->alternateLayout ) {
-		split->moveToLast( panel );
-		split->moveToFirst( pbox );
-	}
 	split->setSizes( dvbConfig->splitSizes );
 }
 
@@ -888,7 +973,7 @@ loop:
 		return;
 	}
 
-	DvbConfigDialog dlg( dvbConfig, mainWidget );
+	DvbConfigDialog dlg( dvbConfig, mainWidget, plug );
 	connect( dlg.dumpBtn, SIGNAL(clicked()), this, SLOT(dumpEvents()) );
 	ret = dlg.exec();
 	disconnect( dlg.dumpBtn, SIGNAL(clicked()), this, SLOT(dumpEvents()) );
@@ -968,17 +1053,21 @@ void DvbPanel::fillChannelList()
 			continue;
 		it = new KListViewItem( channelsCb, QString().sprintf("%04d", chan->num), chan->name );
 		it->setDragEnabled( true );
-		if ( chan->type==1 ) {
-			if ( chan->fta )
-				it->setPixmap( 1, tvcPix );
-			else
-				it->setPixmap( 1, tvPix );
-		}
+		if ( !chan->pix.isNull() )
+			it->setPixmap( 1, chan->pix );
 		else {
-			if ( chan->fta )
-				it->setPixmap( 1, racPix );
-			else
-				it->setPixmap( 1, raPix );
+			if ( chan->type==1 ) {
+				if ( chan->fta )
+					it->setPixmap( 1, tvcPix );
+				else
+					it->setPixmap( 1, tvPix );
+			}
+			else {
+				if ( chan->fta )
+					it->setPixmap( 1, racPix );
+				else
+					it->setPixmap( 1, raPix );
+			}
 		}
 	}
 	trans.clear();
@@ -1084,7 +1173,7 @@ void DvbPanel::checkTimers()
 		t = timers.at(i);
 		if ( t->running )
 			continue;
-		if ( t->begin.date()==cur.date() && t->begin.time().hour()==cur.time().hour() && t->begin.time().minute()==cur.time().minute() ) {
+		if ( t->begin<=cur && cur<t->begin.addSecs(QTime().secsTo(t->duration)) ) {
 			chan = 0;
 			for ( j=0; j<(int)channels.count(); j++ ) {
 				if ( channels.at(j)->name==t->channel ) {
@@ -1386,7 +1475,22 @@ void DvbPanel::showTimers()
 
 
 
-void DvbPanel::newTimer( QString channel, QString name, QDateTime begin, QTime duration)
+int DvbPanel::getSNR( int device )
+{
+	if ( device<0 || device>=(int)dvb.count() )
+		return -1;
+	return dvb.at(device)->getSNR();
+}
+
+
+void DvbPanel::dvbNewTimer( QString name, QString channel, QString datetime, QString duration )
+{
+	newTimer( channel, name, QDateTime::fromString( datetime, Qt::ISODate ), QTime::fromString( duration ), false );
+}
+
+
+
+void DvbPanel::newTimer( QString channel, QString name, QDateTime begin, QTime duration, bool warn )
 {
 	int i, r=0;
 	RecTimer *t;
@@ -1406,7 +1510,8 @@ void DvbPanel::newTimer( QString channel, QString name, QDateTime begin, QTime d
 			r=i+1;
 	}
 	timers.insert( r, rt );
-	KMessageBox::information( 0, i18n("Timer successfully created.") );
+	if ( warn )
+		KMessageBox::information( 0, i18n("Timer successfully created.") );
 }
 
 
@@ -1461,9 +1566,12 @@ void DvbPanel::setShiftLed( bool on )
 	if ( on ) {
 		shiftLed->on();
 		audioComb->setEnabled( false );
+		diskTimer.start( 300000 );
 	}
-	else
+	else {
 		shiftLed->off();
+		diskTimer.stop();
+	}
 }
 
 
@@ -1723,9 +1831,10 @@ void DvbPanel::dvbZap( ChannelDesc *chan )
 	int i;
 	DvbStream *d=0;
 
-	if ( currentFifo.isEmpty() )
+	if ( currentFifo.isEmpty() || isTuning )
 		return;
 
+	isTuning = true;
 	audioComb->setEnabled( false );
 	emit setTimeShiftFilename( "" );
 	for ( i=0; i<(int)dvb.count(); i++ ) {
@@ -1743,6 +1852,22 @@ void DvbPanel::dvbZap( ChannelDesc *chan )
 
 
 
+void DvbPanel::pipeOpened()
+{
+	tuningTimer.start( 1000, true );
+}
+
+
+
+void DvbPanel::setTuning()
+{
+	tuningTimer.stop();
+	stopTuningTimer.stop();
+	isTuning = false;
+}
+
+
+
 void DvbPanel::finalZap( DvbStream *d, ChannelDesc *chan, bool setAudio, int napid )
 {
 	QString s, t;
@@ -1777,6 +1902,7 @@ void DvbPanel::finalZap( DvbStream *d, ChannelDesc *chan, bool setAudio, int nap
 
 	if ( !d1 ) {
 		emit dvbStop();
+		isTuning = false;
 		return;
 	}
 
@@ -1795,6 +1921,9 @@ void DvbPanel::finalZap( DvbStream *d, ChannelDesc *chan, bool setAudio, int nap
 		case DvbStream::ErrCantSetPids :
 			emit showOSD( i18n("Can't set pid(s)"), 5000, 3 );
 			break;
+		case DvbStream::ErrCamUsed :
+			emit showOSD( i18n("No CAM free"), 5000, 3 );
+			break;
 	}
 
 	if ( setAudio ) {
@@ -1815,21 +1944,24 @@ void DvbPanel::finalZap( DvbStream *d, ChannelDesc *chan, bool setAudio, int nap
 	fprintf( stderr, "Tuning delay: %d ms\n", tm.elapsed() );
 	osdMode = 0;
 	if ( ret<1 ) {
-		dvbConfig->lastChannel = chan->num;
 		emit dvbOpen( currentFifo, s.setNum( chan->num)+" - "+chan->name, chan->vpid );
 		showOsdTimer.start( 2000, true );
 		++autocount;
-	}
-	else
-		emit dvbStop();
-	if ( currentFifo==fifoName ) {
-		if ( !fifoName1.isEmpty() )
-			currentFifo = fifoName1;
+		stopTuningTimer.start( 5000, true );
+		if ( currentFifo==fifoName ) {
+			if ( !fifoName1.isEmpty() )
+				currentFifo = fifoName1;
+		}
+		else {
+			if ( !fifoName.isEmpty() )
+				currentFifo = fifoName;
+		}
 	}
 	else {
-		if ( !fifoName.isEmpty() )
-			currentFifo = fifoName;
+		isTuning = false;
+		emit dvbStop();
 	}
+	dvbConfig->lastChannel = chan->num;
 	//QTimer::singleShot( 3000, this, SLOT(playLastChannel()) );
 }
 
@@ -1899,10 +2031,25 @@ bool DvbPanel::getChannelList()
 	QString src="";
 	int ns;
 	KListViewItem *it;
+	QPixmap pix;
 
 	maxChannelNumber = 0;
 	minChannelNumber = -1;
 
+	int sort = dvbConfig->readDvbChanOrder();
+	if ( sort ) {
+		int column = sort&1;
+		int order = sort>>1;
+		switch (order){
+			case 1:
+				channelsCb->setSorting ( column, FALSE );
+				break;
+			case 0:
+				channelsCb->setSorting ( column, TRUE );
+				break;
+		}
+	}
+
 	QFile f( locateLocal("appdata", "channels.dvb" ) );
 	if ( f.open(IO_ReadOnly) ) {
 		QTextStream tt( &f );
@@ -1972,14 +2119,23 @@ bool DvbPanel::getChannelList()
 			s = s.right( s.length()-pos-1 );
 			pos = s.find("|");
 			c = s.left(pos);
-			if ( c.startsWith("T") )
+			if ( c.startsWith("T") ) {
 				chan->tp.type=FE_OFDM;
-			else if ( c.startsWith("C") )
+				chan->tp.source = "Terrestrial";
+			}
+			else if ( c.startsWith("C") ) {
 				chan->tp.type=FE_QAM;
-			else
+				chan->tp.source = "Cable";
+			}
+			else if ( c.startsWith("S") ) {
 				chan->tp.type=FE_QPSK;
-			c.remove( 0, 1 );
-			chan->tp.source = c;
+				c.remove( 0, 1 );
+				chan->tp.source = c;
+			}
+			else {
+				chan->tp.type=FE_ATSC;
+				chan->tp.source = "Atsc";
+			}
 			s = s.right( s.length()-pos-1 );
 			pos = s.find("|");
 			chan->tp.freq = s.left(pos).toULong();
@@ -2020,6 +2176,8 @@ bool DvbPanel::getChannelList()
 				case 64 : chan->tp.modulation = QAM_64; break;
 				case 128 : chan->tp.modulation = QAM_128; break;
 				case 256 : chan->tp.modulation = QAM_256; break;
+				case 108 : chan->tp.modulation = VSB_8; break;
+				case 116 : chan->tp.modulation = VSB_16; break;
 				case -1 : chan->tp.modulation = QAM_AUTO;
 			}
 			s = s.right( s.length()-pos-1 );
@@ -2104,19 +2262,24 @@ bool DvbPanel::getChannelList()
 				continue;
 			}
 
+			chan->pix.load( dvbConfig->dvbConfigIconsDir+chan->name );
 			it = new KListViewItem( channelsCb, QString().sprintf("%04d", chan->num), chan->name );
 			it->setDragEnabled( true );
-			if ( chan->type==1 ) {
-				if ( chan->fta )
-					it->setPixmap( 1, tvcPix );
-				else
-					it->setPixmap( 1, tvPix );
-			}
+			if ( !chan->pix.isNull() )
+				it->setPixmap( 1, chan->pix );
 			else {
-				if ( chan->fta )
-					it->setPixmap( 1, racPix );
-				else
-					it->setPixmap( 1, raPix );
+				if ( chan->type==1 ) {
+					if ( chan->fta )
+						it->setPixmap( 1, tvcPix );
+					else
+						it->setPixmap( 1, tvPix );
+				}
+				else {
+					if ( chan->fta )
+						it->setPixmap( 1, racPix );
+					else
+						it->setPixmap( 1, raPix );
+				}
 			}
 			channels.append( chan );
 			if ( chan->num<=0 )
@@ -2182,11 +2345,11 @@ bool DvbPanel::saveChannelList()
 			tt<< s.setNum(chan->sid)+"|";
 			tt<< s.setNum(chan->tp.tsid)+"|";
 			switch ( chan->tp.type ) {
-				case FE_QPSK : tt<< "S"; break;
-				case FE_QAM : tt<< "C"; break;
-				case FE_OFDM : tt<< "T";
+				case FE_QPSK : tt<< "S"; tt<< chan->tp.source; break;
+				case FE_QAM : tt<< "Cable"; break;
+				case FE_OFDM : tt<< "Terrestrial"; break;
+				case FE_ATSC : tt<< "Atsc"; break;
 			}
-			tt<< chan->tp.source;
 			tt<< "|";
 			tt<< s.setNum(chan->tp.freq)+"|";
 			tt<< s.setNum(chan->tp.sr)+"|";
@@ -2218,6 +2381,8 @@ bool DvbPanel::saveChannelList()
 				case QAM_64 : tt<< "64|"; break;
 				case QAM_128 : tt<< "128|"; break;
 				case QAM_256 : tt<< "256|"; break;
+				case VSB_8 : tt<< "108|"; break;
+				case VSB_16 : tt<< "116|"; break;
 				case QAM_AUTO : tt<< "-1|";
 			}
 			switch ( chan->tp.coderateL ) {
@@ -2381,15 +2546,26 @@ bool DvbPanel::close()
 
 
 
-DvbPanel::~DvbPanel()
+void DvbPanel::saveConfig()
 {
 	saveTimerList();
 	saveChannelList();
 	dvbConfig->splitSizes = split->sizes();
-	delete dvbConfig;
+	dvbConfig->saveDvbChanOrder( channelsCb->sortOrder(), channelsCb->sortColumn() );
+	dvbConfig->saveConfig();
+	//delete dvbConfig;
+}
+
+
+DvbPanel::~DvbPanel()
+{
 	dvb.clear();
 	delete rtp;
 	delete cleaner;
+	if ( plug ) {
+		KService::Ptr service = KService::serviceByDesktopName( plugName );
+		KLibLoader::self()->unloadLibrary( service->library().ascii() );
+	}
 }
 
 #include "dvbpanel.moc"
