@@ -1,20 +1,22 @@
-/***************************************************************************
-                          dvbsi.cpp  -  description
-                             -------------------
-    begin                : 2004
-    copyright            : (C) 2003-2005 by Christophe Thommeret
-    email                : hftom@free.fr
-    last modified        : $Date: 2005/08/27 00:35:28 $ by $Author: hftom $
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/*
+ * dvbsi.cpp
+ *
+ * Copyright (C) 2003-2005 Christophe Thommeret <hftom@free.fr>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+ */
 
 #include <stdio.h>
 #include <unistd.h>
@@ -46,6 +48,7 @@ NitSection::NitSection( QPtrList<Transponder> *tp, bool *end, bool *ok, int anum
 
 NitSection::~NitSection()
 {
+	stop();
 }
 
 
@@ -540,13 +543,14 @@ bool DVBsi::tablePAT( unsigned char *buf )
 
 
 
-bool DVBsi::getSection( int pid, int tid, int timeout )
+bool DVBsi::getSection( int pid, int tid, int timeout, int sid )
 {
 	unsigned char buf[4096];
 	int i, n=0;
 	int skip=0;
 	int last=1, current=0, loop=0;
-	QValueList<int> list;
+	QValueList<int> list, sidList;
+	int cursid;
 
 	if ( !setFilter( pid, tid, timeout ) )
 		return false;
@@ -569,6 +573,12 @@ bool DVBsi::getSection( int pid, int tid, int timeout )
 			return false;
 		}
 
+		cursid = getBits(buf+3,0,16);
+		if ( sid && cursid!=sid && !sidList.contains(cursid) ) {
+			sidList.append( cursid );
+			continue;
+		}
+
 		last = getBits(buf,56,8);
 		current = getBits(buf,48,8);
 		for ( i=0; i<(int)list.count(); i++ ) {
@@ -615,13 +625,13 @@ void DVBsi::stop()
 
 	isRunning = false;
 	if ( !wait(2000) ) {
-		terminate();
-		wait();
 		if ( ns ) {
 			ns->stop();
 			delete ns;
 			ns = 0;
 		}
+		terminate();
+		wait();
 		if ( scanMode<2 )
 			dvb->stopScan();
 	}
@@ -678,7 +688,7 @@ void DVBsi::run()
 	ChannelDesc chan;
 	Transponder trans;
 	unsigned short tsid;
-	bool nitEnded, ok;
+	bool nitEnded=true, ok;
 
 	progressTransponder = 0;
 
@@ -690,7 +700,7 @@ void DVBsi::run()
 				return;
 			}
 			chan.tp = *transponders.at(i);
-			usleep(100000);
+			usleep( 100000 );
 			if ( !dvb->tuneDvb( &chan, false ) ) {
 				for ( k=0; k<(int)chan.tp.freqlist.count(); k++ ) {
 					chan.tp.freq = chan.tp.freqlist[k];
@@ -700,19 +710,23 @@ void DVBsi::run()
 						transponders.at(i)->freq = chan.tp.freq;
 						break;
 					}
+					else
+						usleep( 400000 );
 				}
 				if ( k>-1 ) {
 					fprintf(stderr,"dvbsi: Cant tune DVB\n");
 					progressTransponder++;
-					sleep(1);
+					usleep( 400000 );
 					continue;
 				}
 			}
 
 			indexChannels = j;
 			fprintf(stderr,"Transponders: %d/%d\n", i+1, transponders.count() );
-			nitEnded = false;
-			ns = new NitSection( &transponders, &nitEnded, &ok, adapter, tuner ); //NIT
+			if ( scanMode ) {
+				nitEnded = false;
+				ns = new NitSection( &transponders, &nitEnded, &ok, adapter, tuner ); //NIT
+			}
 			getSection( 0x11, 0x42 );         //SDT
 			if ( !isRunning ) {
 				out();
@@ -728,11 +742,12 @@ void DVBsi::run()
 					out();
 					return;
 				}
-				getSection( channels.at( j )->pmtpid, 0x02 );    //PMTs
+				getSection( channels.at( j )->pmtpid, 0x02, 5000, channels.at( j )->sid );    //PMTs
 				tsid = channels.at(j)->tp.tsid;
 				channels.at(j)->tp = *transponders.at(i);
 				if ( channels.at(j)->tp.tsid==0 )
 					channels.at(j)->tp.tsid = tsid;
+				channels.at(j)->tp.snr = dvb->getSNR();
 				channels.at(j)->completed = 1;
 			}
 			while( !nitEnded ) {
@@ -743,8 +758,10 @@ void DVBsi::run()
 				else
 					msleep( 50 );
 			}
-			delete ns;
-			ns = 0;
+			if ( scanMode ) {
+				delete ns;
+				ns = 0;
+			}
 			dvb->stopScan();
 			progressTransponder++;
 		}
@@ -764,10 +781,11 @@ void DVBsi::run()
 				out(false);
 				return;
 			}
-			getSection( channels.at( j )->pmtpid, 0x02 );    //PMTs
+			getSection( channels.at( j )->pmtpid, 0x02, 5000, channels.at( j )->sid );    //PMTs
 			tsid = channels.at(j)->tp.tsid;
 			channels.at(j)->tp = trans;
 			channels.at(j)->tp.tsid = tsid;
+			channels.at(j)->tp.snr = dvb->getSNR();
 			channels.at(j)->completed = 1;
 		}
 	}
